@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,30 +18,17 @@ import sys
 
 import ludwig.contrib
 
-ludwig.contrib.contrib_import()
 
+class CLI:
+    """CLI describes a command line interface for interacting with Ludwig.
 
-class CLI(object):
-    """CLI describes a command line interface for interacting with Ludwig, there
-    are several different functions that can be performed. These functions are:
-    - experiment - run an experiment using ludwig
-    - predict - Given a list of $hat{y}$ values, compute $d(\\hat{y}, y) under a
-      specified metric
-    - train - trains a model on the input file specified to it
-    - visualize - Analysis of the results for the model on the dataset and
-      presents a variety of plots to understand and evaluate the results
-    - collect_weights - Collects the weights for a pretrained model as a tensor
-      representation
-    - collect_activations - For each datapoint, there exists a corresponding
-      tensor representation which are collected through this method
-    - hyperopt - Performs an hyper-parameter search
-      with a given strategy and parameters
+    Functions are described below.
     """
 
     def __init__(self):
         parser = argparse.ArgumentParser(
-            description='ludwig cli runner',
-            usage='''ludwig <command> [<args>]
+            description="ludwig cli runner",
+            usage="""ludwig <command> [<args>]
 
 Available sub-commands:
    train                 Trains a model
@@ -55,17 +41,22 @@ Available sub-commands:
    collect_summary       Prints names of weights and layers activations to use with other collect commands
    collect_weights       Collects tensors containing a pretrained model weights
    collect_activations   Collects tensors for each datapoint using a pretrained model
-   export_savedmodel     Exports Ludwig models to SavedModel
+   datasets              Downloads and lists Ludwig-ready datasets
+   export_torchscript    Exports Ludwig models to Torchscript
    export_neuropod       Exports Ludwig models to Neuropod
+   export_mlflow         Exports Ludwig models to MLflow
    preprocess            Preprocess data and saves it into HDF5 and JSON format
-   synthesize_dataset    Creates synthetic data for tesing purposes
-''')
-        parser.add_argument('command', help='Subcommand to run')
+   synthesize_dataset    Creates synthetic data for testing purposes
+   init_config           Initialize a user config from a dataset and targets
+   render_config         Renders the fully populated config with all defaults set
+""",
+        )
+        parser.add_argument("command", help="Subcommand to run")
         # parse_args defaults to [1:] for args, but you need to
         # exclude the rest of the args too, or validation will fail
         args = parser.parse_args(sys.argv[1:2])
         if not hasattr(self, args.command):
-            print('Unrecognized command')
+            print("Unrecognized command")
             parser.print_help()
             exit(1)
         # use dispatch pattern to invoke method with same name
@@ -73,78 +64,99 @@ Available sub-commands:
 
     def train(self):
         from ludwig import train
-        ludwig.contrib.contrib_command("train", *sys.argv)
+
         train.cli(sys.argv[2:])
 
     def predict(self):
         from ludwig import predict
-        ludwig.contrib.contrib_command("predict", *sys.argv)
+
         predict.cli(sys.argv[2:])
 
     def evaluate(self):
         from ludwig import evaluate
-        ludwig.contrib.contrib_command("evaluate", *sys.argv)
+
         evaluate.cli(sys.argv[2:])
 
     def experiment(self):
         from ludwig import experiment
-        ludwig.contrib.contrib_command("experiment", *sys.argv)
+
         experiment.cli(sys.argv[2:])
 
     def hyperopt(self):
         from ludwig import hyperopt_cli
-        ludwig.contrib.contrib_command("hyperopt", *sys.argv)
+
         hyperopt_cli.cli(sys.argv[2:])
 
     def serve(self):
         from ludwig import serve
-        ludwig.contrib.contrib_command("serve", *sys.argv)
+
         serve.cli(sys.argv[2:])
 
     def visualize(self):
         from ludwig import visualize
-        ludwig.contrib.contrib_command("visualize", *sys.argv)
+
         visualize.cli(sys.argv[2:])
 
     def collect_summary(self):
         from ludwig import collect
-        ludwig.contrib.contrib_command('collect_summary', *sys.argv)
+
         collect.cli_collect_summary(sys.argv[2:])
 
     def collect_weights(self):
         from ludwig import collect
-        ludwig.contrib.contrib_command("collect_weights", *sys.argv)
+
         collect.cli_collect_weights(sys.argv[2:])
 
     def collect_activations(self):
         from ludwig import collect
-        ludwig.contrib.contrib_command("collect_activations", *sys.argv)
+
         collect.cli_collect_activations(sys.argv[2:])
 
-    def export_savedmodel(self):
+    def export_torchscript(self):
         from ludwig import export
-        ludwig.contrib.contrib_command("export_savedmodel", *sys.argv)
-        export.cli_export_savedmodel(sys.argv[2:])
+
+        export.cli_export_torchscript(sys.argv[2:])
 
     def export_neuropod(self):
         from ludwig import export
-        ludwig.contrib.contrib_command("export_neuropod", *sys.argv)
+
         export.cli_export_neuropod(sys.argv[2:])
 
+    def export_mlflow(self):
+        from ludwig import export
+
+        export.cli_export_mlflow(sys.argv[2:])
+
     def preprocess(self):
         from ludwig import preprocess
-        ludwig.contrib.contrib_command("preprocess", *sys.argv)
+
         preprocess.cli(sys.argv[2:])
 
     def synthesize_dataset(self):
         from ludwig.data import dataset_synthesizer
-        ludwig.contrib.contrib_command("synthesize_dataset", *sys.argv)
+
         dataset_synthesizer.cli(sys.argv[2:])
 
+    def init_config(self):
+        from ludwig import automl
+
+        automl.cli_init_config(sys.argv[2:])
+
+    def render_config(self):
+        from ludwig.utils import defaults
+
+        defaults.cli_render_config(sys.argv[2:])
+
+    def datasets(self):
+        from ludwig import datasets
+
+        datasets.cli(sys.argv[2:])
+
 
 def main():
+    ludwig.contrib.preload(sys.argv)
     CLI()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
