@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,8 +13,7 @@
 # limitations under the License.
 # ==============================================================================
 
-"""
-All contrib classes must have the following methods:
+"""All contrib classes must have the following methods:
 
 - import_call: Run on import.
 
@@ -36,14 +34,15 @@ method with `pass`, or just don't implement the method.
 """
 
 # Contributors, import your class here:
-from .comet import Comet
-from .wandb import Wandb
+from .comet import CometCallback
+from .mlflow import MlflowCallback
+from .wandb import WandbCallback
+from .whylogs import WhyLogsCallback
 
 contrib_registry = {
     # Contributors, add your class here:
-    'classes': {
-        'comet': Comet,
-        'wandb': Wandb,
-    },
-    'instances': [],
+    "comet": CometCallback,
+    "wandb": WandbCallback,
+    "mlflow": MlflowCallback,
+    "whylogs": WhyLogsCallback,
 }
