@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,13 +14,21 @@
 # limitations under the License.
 # ==============================================================================
 
-LUDWIG_VERSION = '0.3.3'
+LUDWIG_VERSION = "0.5"
 
-MODEL_WEIGHTS_FILE_NAME = 'model_weights'
-MODEL_HYPERPARAMETERS_FILE_NAME = 'model_hyperparameters.json'
-TRAIN_SET_METADATA_FILE_NAME = 'training_set_metadata.json'
-TRAINING_PROGRESS_TRACKER_FILE_NAME = 'training_progress.json'
-TRAINING_CHECKPOINTS_DIR_PATH = 'training_checkpoints'
+INFERENCE_MODULE_FILE_NAME = "inference_module"
+MODEL_WEIGHTS_FILE_NAME = "model_weights"
+MODEL_HYPERPARAMETERS_FILE_NAME = "model_hyperparameters.json"
+TRAIN_SET_METADATA_FILE_NAME = "training_set_metadata.json"
+TRAINING_PROGRESS_TRACKER_FILE_NAME = "training_progress.json"
+TRAINING_CHECKPOINTS_DIR_PATH = "training_checkpoints"
+
+TEST_STATISTICS_FILE_NAME = "test_statistics.json"
+
+PREDICTIONS_PARQUET_FILE_NAME = "predictions.parquet"
+PREDICTIONS_SHAPES_FILE_NAME = "predictions.shapes.json"
+
+TRAINING_PREPROC_FILE_NAME = "training.hdf5"
 
 DISABLE_PROGRESSBAR = False
 
