@@ -1,54 +1,63 @@
 import logging
-import os
 from pprint import pformat
-from typing import Union, List
+from typing import List, Union
 
 import pandas as pd
 import yaml
 
-from ludwig.constants import HYPEROPT, TRAINING, VALIDATION, TEST, COMBINED, \
-    LOSS, TYPE
+from ludwig.api import LudwigModel
+from ludwig.backend import Backend, initialize_backend, LocalBackend
+from ludwig.callbacks import Callback
+from ludwig.constants import COMBINED, EXECUTOR, HYPEROPT, LOSS, MINIMIZE, SAMPLER, TEST, TRAINING, TYPE, VALIDATION
 from ludwig.features.feature_registries import output_type_registry
-from ludwig.hyperopt.execution import get_build_hyperopt_executor
-from ludwig.hyperopt.sampling import get_build_hyperopt_sampler
-from ludwig.hyperopt.utils import update_hyperopt_params_with_defaults, \
-    print_hyperopt_results, save_hyperopt_stats
+from ludwig.hyperopt.execution import executor_registry, get_build_hyperopt_executor, RayTuneExecutor
+from ludwig.hyperopt.results import HyperoptResults
+from ludwig.hyperopt.sampling import get_build_hyperopt_sampler, sampler_registry
+from ludwig.hyperopt.utils import print_hyperopt_results, save_hyperopt_stats, should_tune_preprocessing
 from ludwig.utils.defaults import default_random_seed, merge_with_defaults
-from ludwig.utils.horovod_utils import is_on_master
-from ludwig.utils.misc_utils import get_from_registry
+from ludwig.utils.fs_utils import makedirs, open_file
+from ludwig.utils.misc_utils import get_class_attributes, get_from_registry, set_default_value, set_default_values
+
+try:
+    from ludwig.backend.ray import RayBackend
+except ImportError:
+
+    class RayBackend:
+        pass
+
 
 logger = logging.getLogger(__name__)
 
 
 def hyperopt(
-        config: Union[str, dict],
-        dataset: Union[str, dict, pd.DataFrame] = None,
-        training_set: Union[str, dict, pd.DataFrame] = None,
-        validation_set: Union[str, dict, pd.DataFrame] = None,
-        test_set: Union[str, dict, pd.DataFrame] = None,
-        training_set_metadata: Union[str, dict] = None,
-        data_format: str = None,
-        experiment_name: str = 'hyperopt',
-        model_name: str = 'run',
-        skip_save_training_description: bool = False,
-        skip_save_training_statistics: bool = False,
-        skip_save_model: bool = False,
-        skip_save_progress: bool = False,
-        skip_save_log: bool = False,
-        skip_save_processed_input: bool = False,
-        skip_save_unprocessed_output: bool = False,
-        skip_save_predictions: bool = False,
-        skip_save_eval_stats: bool = False,
-        skip_save_hyperopt_statistics: bool = False,
-        output_directory: str = 'results',
-        gpus: Union[str, int, List[int]] = None,
-        gpu_memory_limit: int = None,
-        allow_parallel_threads: bool = True,
-        use_horovod: bool = None,
-        random_seed: int = default_random_seed,
-        debug: bool = False,
-        **kwargs,
-) -> List[dict]:
+    config: Union[str, dict],
+    dataset: Union[str, dict, pd.DataFrame] = None,
+    training_set: Union[str, dict, pd.DataFrame] = None,
+    validation_set: Union[str, dict, pd.DataFrame] = None,
+    test_set: Union[str, dict, pd.DataFrame] = None,
+    training_set_metadata: Union[str, dict] = None,
+    data_format: str = None,
+    experiment_name: str = "hyperopt",
+    model_name: str = "run",
+    skip_save_training_description: bool = False,
+    skip_save_training_statistics: bool = False,
+    skip_save_model: bool = False,
+    skip_save_progress: bool = False,
+    skip_save_log: bool = False,
+    skip_save_processed_input: bool = True,
+    skip_save_unprocessed_output: bool = False,
+    skip_save_predictions: bool = False,
+    skip_save_eval_stats: bool = False,
+    skip_save_hyperopt_statistics: bool = False,
+    output_directory: str = "results",
+    gpus: Union[str, int, List[int]] = None,
+    gpu_memory_limit: int = None,
+    allow_parallel_threads: bool = True,
+    callbacks: List[Callback] = None,
+    backend: Union[Backend, str] = None,
+    random_seed: int = default_random_seed,
+    **kwargs,
+) -> HyperoptResults:
     """This method performs an hyperparameter optimization.
 
     # Inputs
@@ -132,19 +141,22 @@ def hyperopt(
     :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
         to use multithreading parallelism to improve performance at
         the cost of determinism.
-    :param use_horovod: (bool, default: `None`) flag for using horovod.
+    :param callbacks: (list, default: `None`) a list of
+        `ludwig.callbacks.Callback` objects that provide hooks into the
+        Ludwig pipeline.
+    :param backend: (Union[Backend, str]) `Backend` or string name
+        of backend to use to execute preprocessing / training steps.
     :param random_seed: (int: default: 42) random seed used for weights
         initialization, splits and any other random function.
-    :param debug: (bool, default: `False) if `True` turns on `tfdbg` with
-        `inf_or_nan` checks.
 
     # Return
 
-    :return: (List[dict]) The results for the hyperparameter optimization
+    :return: (List[dict]) List of results for each trial, ordered by
+        descending performance on the target metric.
     """
     # check if config is a path or a dict
     if isinstance(config, str):  # assume path
-        with open(config, 'r') as def_file:
+        with open_file(config, "r") as def_file:
             config_dict = yaml.safe_load(def_file)
     else:
         config_dict = config
@@ -153,16 +165,15 @@ def hyperopt(
     config = merge_with_defaults(config_dict)
 
     if HYPEROPT not in config:
-        raise ValueError(
-            "Hyperopt Section not present in config"
-        )
+        raise ValueError("Hyperopt Section not present in config")
 
     hyperopt_config = config["hyperopt"]
+
     update_hyperopt_params_with_defaults(hyperopt_config)
 
     # print hyperopt config
     logger.info(pformat(hyperopt_config, indent=4))
-    logger.info('\n')
+    logger.info("\n")
 
     sampler = hyperopt_config["sampler"]
     executor = hyperopt_config["executor"]
@@ -176,68 +187,50 @@ def hyperopt(
     # check validity of output_feature / metric/ split combination
     ######################
     if split == TRAINING:
-        if training_set is None and (
-                config['preprocessing']['split_probabilities'][0]
-                <= 0):
+        if training_set is None and (config["preprocessing"]["split_probabilities"][0] <= 0):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
-                'was not provided, '
-                'or the split amount specified in the preprocessing section '
-                'of the config is not greater than 0'.format(split)
+                "was not provided, "
+                "or the split amount specified in the preprocessing section "
+                "of the config is not greater than 0".format(split)
             )
     elif split == VALIDATION:
-        if validation_set is None and (
-                config['preprocessing']['split_probabilities'][1]
-                <= 0):
+        if validation_set is None and (config["preprocessing"]["split_probabilities"][1] <= 0):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
-                'was not provided, '
-                'or the split amount specified in the preprocessing section '
-                'of the config is not greater than 0'.format(split)
+                "was not provided, "
+                "or the split amount specified in the preprocessing section "
+                "of the config is not greater than 0".format(split)
             )
     elif split == TEST:
-        if test_set is None and (
-                config['preprocessing']['split_probabilities'][2]
-                <= 0):
+        if test_set is None and (config["preprocessing"]["split_probabilities"][2] <= 0):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
-                'was not provided, '
-                'or the split amount specified in the preprocessing section '
-                'of the config is not greater than 0'.format(split)
+                "was not provided, "
+                "or the split amount specified in the preprocessing section "
+                "of the config is not greater than 0".format(split)
             )
     else:
         raise ValueError(
-            'unrecognized hyperopt split "{}". '
-            'Please provide one of: {}'.format(
-                split, {TRAINING, VALIDATION, TEST}
-            )
+            'unrecognized hyperopt split "{}". ' "Please provide one of: {}".format(split, {TRAINING, VALIDATION, TEST})
         )
     if output_feature == COMBINED:
         if metric != LOSS:
-            raise ValueError(
-                'The only valid metric for "combined" output feature is "loss"'
-            )
+            raise ValueError('The only valid metric for "combined" output feature is "loss"')
     else:
-        output_feature_names = set(
-            of['name'] for of in config['output_features']
-        )
+        output_feature_names = {of["name"] for of in config["output_features"]}
         if output_feature not in output_feature_names:
             raise ValueError(
                 'The output feature specified for hyperopt "{}" '
-                'cannot be found in the config. '
-                'Available ones are: {} and "combined"'.format(
-                    output_feature, output_feature_names
-                )
+                "cannot be found in the config. "
+                'Available ones are: {} and "combined"'.format(output_feature, output_feature_names)
             )
 
         output_feature_type = None
-        for of in config['output_features']:
-            if of['name'] == output_feature:
+        for of in config["output_features"]:
+            if of["name"] == output_feature:
                 output_feature_type = of[TYPE]
-        feature_class = get_from_registry(
-            output_feature_type,
-            output_type_registry
-        )
+        feature_class = get_from_registry(output_feature_type, output_type_registry)
         if metric not in feature_class.metric_functions:
             # todo v0.4: allow users to specify also metrics from the overall
             #  and per class metrics from the trainign stats and in general
@@ -245,20 +238,63 @@ def hyperopt(
             raise ValueError(
                 'The specified metric for hyperopt "{}" is not a valid metric '
                 'for the specified output feature "{}" of type "{}". '
-                'Available metrics are: {}'.format(
-                    metric,
-                    output_feature,
-                    output_feature_type,
-                    feature_class.metric_functions.keys()
+                "Available metrics are: {}".format(
+                    metric, output_feature, output_feature_type, feature_class.metric_functions.keys()
                 )
             )
 
-    hyperopt_sampler = get_build_hyperopt_sampler(
-        sampler[TYPE]
-    )(goal, parameters, **sampler)
-    hyperopt_executor = get_build_hyperopt_executor(
-        executor[TYPE]
-    )(hyperopt_sampler, output_feature, metric, split, **executor)
+    hyperopt_sampler = get_build_hyperopt_sampler(sampler[TYPE])(goal, parameters, **sampler)
+
+    hyperopt_executor = get_build_hyperopt_executor(executor[TYPE])(
+        hyperopt_sampler, output_feature, metric, split, **executor
+    )
+
+    # Explicitly default to a local backend to avoid picking up Ray or Horovod
+    # backend from the environment.
+    backend = backend or config_dict.get("backend") or "local"
+    backend = initialize_backend(backend)
+    if not (
+        isinstance(backend, LocalBackend)
+        or (isinstance(hyperopt_executor, RayTuneExecutor) and isinstance(backend, RayBackend))
+    ):
+        raise ValueError(
+            "Hyperopt requires using a `local` backend at this time, or " "`ray` backend with `ray` executor."
+        )
+
+    for callback in callbacks or []:
+        callback.on_hyperopt_init(experiment_name)
+
+    if not should_tune_preprocessing(config):
+        # preprocessing is not being tuned, so generate it once before starting trials
+        for callback in callbacks or []:
+            callback.on_hyperopt_preprocessing_start(experiment_name)
+
+        model = LudwigModel(
+            config=config,
+            backend=backend,
+            gpus=gpus,
+            gpu_memory_limit=gpu_memory_limit,
+            allow_parallel_threads=allow_parallel_threads,
+            callbacks=callbacks,
+        )
+
+        training_set, validation_set, test_set, training_set_metadata = model.preprocess(
+            dataset=dataset,
+            training_set=training_set,
+            validation_set=validation_set,
+            test_set=test_set,
+            training_set_metadata=training_set_metadata,
+            data_format=data_format,
+            skip_save_processed_input=skip_save_processed_input,
+            random_seed=random_seed,
+        )
+        dataset = None
+
+        for callback in callbacks or []:
+            callback.on_hyperopt_preprocessing_end(experiment_name)
+
+    for callback in callbacks or []:
+        callback.on_hyperopt_start(experiment_name)
 
     hyperopt_results = hyperopt_executor.execute(
         config,
@@ -285,27 +321,57 @@ def hyperopt(
         gpus=gpus,
         gpu_memory_limit=gpu_memory_limit,
         allow_parallel_threads=allow_parallel_threads,
-        use_horovod=use_horovod,
+        callbacks=callbacks,
+        backend=backend,
         random_seed=random_seed,
-        debug=debug,
-        **kwargs
+        **kwargs,
     )
 
-    if is_on_master():
+    if backend.is_coordinator():
         print_hyperopt_results(hyperopt_results)
 
         if not skip_save_hyperopt_statistics:
-            if not os.path.exists(output_directory):
-                os.makedirs(output_directory)
+            makedirs(output_directory, exist_ok=True)
 
             hyperopt_stats = {
-                'hyperopt_config': hyperopt_config,
-                'hyperopt_results': hyperopt_results
+                "hyperopt_config": hyperopt_config,
+                "hyperopt_results": [t.to_dict() for t in hyperopt_results.ordered_trials],
             }
 
             save_hyperopt_stats(hyperopt_stats, output_directory)
-            logger.info('Hyperopt stats saved to: {}'.format(output_directory))
+            logger.info(f"Hyperopt stats saved to: {output_directory}")
 
-    logger.info('Finished hyperopt')
+    for callback in callbacks or []:
+        callback.on_hyperopt_end(experiment_name)
+        callback.on_hyperopt_finish(experiment_name)
+
+    logger.info("Finished hyperopt")
 
     return hyperopt_results
+
+
+def update_hyperopt_params_with_defaults(hyperopt_params):
+    set_default_value(hyperopt_params, SAMPLER, {})
+    set_default_value(hyperopt_params, EXECUTOR, {})
+    set_default_value(hyperopt_params, "split", VALIDATION)
+    set_default_value(hyperopt_params, "output_feature", COMBINED)
+    set_default_value(hyperopt_params, "metric", LOSS)
+    set_default_value(hyperopt_params, "goal", MINIMIZE)
+
+    set_default_values(hyperopt_params[SAMPLER], {TYPE: "random"})
+
+    sampler = get_from_registry(hyperopt_params[SAMPLER][TYPE], sampler_registry)
+    sampler_defaults = {k: v for k, v in sampler.__dict__.items() if k in get_class_attributes(sampler)}
+    set_default_values(
+        hyperopt_params[SAMPLER],
+        sampler_defaults,
+    )
+
+    set_default_values(hyperopt_params[EXECUTOR], {TYPE: "serial"})
+
+    executor = get_from_registry(hyperopt_params[EXECUTOR][TYPE], executor_registry)
+    executor_defaults = {k: v for k, v in executor.__dict__.items() if k in get_class_attributes(executor)}
+    set_default_values(
+        hyperopt_params[EXECUTOR],
+        executor_defaults,
+    )
