@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,56 +19,54 @@ import sys
 from typing import List, Union
 
 import pandas as pd
-import yaml
 
 from ludwig.api import LudwigModel
-from ludwig.contrib import contrib_command, contrib_import
+from ludwig.backend import ALL_BACKENDS, Backend, initialize_backend
+from ludwig.callbacks import Callback
+from ludwig.constants import HYPEROPT, HYPEROPT_WARNING
+from ludwig.contrib import add_contrib_callback_args
 from ludwig.globals import LUDWIG_VERSION
+from ludwig.utils.data_utils import load_config_from_str, load_yaml
 from ludwig.utils.defaults import default_random_seed
-from ludwig.utils.horovod_utils import is_on_master, set_on_master
-from ludwig.utils.misc_utils import check_which_config
-from ludwig.utils.print_utils import logging_level_registry, print_ludwig
+from ludwig.utils.print_utils import logging_level_registry, print_ludwig, query_yes_no
 
 logger = logging.getLogger(__name__)
 
 
 def train_cli(
-        config: dict = None,
-        config_file: str = None,
-        dataset: Union[str, dict, pd.DataFrame] = None,
-        training_set: Union[str, dict, pd.DataFrame] = None,
-        validation_set: Union[str, dict, pd.DataFrame] = None,
-        test_set: Union[str, dict, pd.DataFrame] = None,
-        training_set_metadata: Union[str, dict] = None,
-        data_format: str = None,
-        experiment_name: str = 'api_experiment',
-        model_name: str = 'run',
-        model_load_path: str = None,
-        model_resume_path: str = None,
-        skip_save_training_description: bool = False,
-        skip_save_training_statistics: bool = False,
-        skip_save_model: bool = False,
-        skip_save_progress: bool = False,
-        skip_save_log: bool = False,
-        skip_save_processed_input: bool = False,
-        output_directory: str = 'results',
-        gpus: Union[str, int, List[int]] = None,
-        gpu_memory_limit: int = None,
-        allow_parallel_threads: bool = True,
-        use_horovod: bool = None,
-        random_seed: int = default_random_seed,
-        logging_level: int =logging.INFO,
-        debug: bool = False,
-        **kwargs
+    config: Union[str, dict] = None,
+    dataset: Union[str, dict, pd.DataFrame] = None,
+    training_set: Union[str, dict, pd.DataFrame] = None,
+    validation_set: Union[str, dict, pd.DataFrame] = None,
+    test_set: Union[str, dict, pd.DataFrame] = None,
+    training_set_metadata: Union[str, dict] = None,
+    data_format: str = None,
+    experiment_name: str = "api_experiment",
+    model_name: str = "run",
+    model_load_path: str = None,
+    model_resume_path: str = None,
+    skip_save_training_description: bool = False,
+    skip_save_training_statistics: bool = False,
+    skip_save_model: bool = False,
+    skip_save_progress: bool = False,
+    skip_save_log: bool = False,
+    skip_save_processed_input: bool = False,
+    output_directory: str = "results",
+    gpus: Union[str, int, List[int]] = None,
+    gpu_memory_limit: int = None,
+    allow_parallel_threads: bool = True,
+    callbacks: List[Callback] = None,
+    backend: Union[Backend, str] = None,
+    random_seed: int = default_random_seed,
+    logging_level: int = logging.INFO,
+    **kwargs
 ) -> None:
     """*train* defines the entire training procedure used by Ludwig's
     internals. Requires most of the parameters that are taken into the model.
     Builds a full ludwig model and performs the training.
 
-    :param config: (dict) config which defines the different
-        parameters of the model, features, preprocessing and training.
-    :param config_file: (str, default: `None`) the filepath string
-        that specifies the config.  It is a yaml file.
+    :param config: (Union[str, dict]) in-memory representation of
+            config or string path to a YAML config file.
     :param dataset: (Union[str, dict, pandas.DataFrame], default: `None`)
         source containing the entire dataset to be used for training.
         If it has a split column, it will be used for splitting (0 for train,
@@ -144,37 +141,42 @@ def train_cli(
     :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
         to use multithreading parallelism to improve performance at
         the cost of determinism.
-    :param use_horovod: (bool, default: `None`) flag for using horovod.
+    :param callbacks: (list, default: `None`) a list of
+        `ludwig.callbacks.Callback` objects that provide hooks into the
+        Ludwig pipeline.
+    :param backend: (Union[Backend, str]) `Backend` or string name
+        of backend to use to execute preprocessing / training steps.
     :param random_seed: (int: default: 42) random seed used for weights
         initialization, splits and any other random function.
-    :param debug: (bool, default: `False) if `True` turns on `tfdbg` with
-        `inf_or_nan` checks.
     :param logging_level: (int) Log level that will be sent to stderr.
 
     # Return
 
     :return: (`None`)
     """
-    config = check_which_config(config,
-                                config_file)
+    if HYPEROPT in config:
+        if not query_yes_no(HYPEROPT_WARNING):
+            exit(1)
 
     if model_load_path:
         model = LudwigModel.load(
             model_load_path,
             logging_level=logging_level,
-            use_horovod=use_horovod,
+            backend=backend,
             gpus=gpus,
             gpu_memory_limit=gpu_memory_limit,
             allow_parallel_threads=allow_parallel_threads,
+            callbacks=callbacks,
         )
     else:
         model = LudwigModel(
             config=config,
             logging_level=logging_level,
-            use_horovod=use_horovod,
+            backend=backend,
             gpus=gpus,
             gpu_memory_limit=gpu_memory_limit,
             allow_parallel_threads=allow_parallel_threads,
+            callbacks=callbacks,
         )
     model.train(
         dataset=dataset,
@@ -194,78 +196,72 @@ def train_cli(
         skip_save_processed_input=skip_save_processed_input,
         output_directory=output_directory,
         random_seed=random_seed,
-        debug=debug,
     )
 
 
 def cli(sys_argv):
     parser = argparse.ArgumentParser(
-        description='This script trains a model',
-        prog='ludwig train',
-        usage='%(prog)s [options]'
+        description="This script trains a model", prog="ludwig train", usage="%(prog)s [options]"
     )
 
     # ----------------------------
     # Experiment naming parameters
     # ----------------------------
-    parser.add_argument(
-        '--output_directory',
-        type=str,
-        default='results',
-        help='directory that contains the results'
-    )
-    parser.add_argument(
-        '--experiment_name',
-        type=str,
-        default='experiment',
-        help='experiment name'
-    )
-    parser.add_argument(
-        '--model_name',
-        type=str,
-        default='run',
-        help='name for the model'
-    )
+    parser.add_argument("--output_directory", type=str, default="results", help="directory that contains the results")
+    parser.add_argument("--experiment_name", type=str, default="experiment", help="experiment name")
+    parser.add_argument("--model_name", type=str, default="run", help="name for the model")
 
     # ---------------
     # Data parameters
     # ---------------
     parser.add_argument(
-        '--dataset',
-        help='input data file path. '
-             'If it has a split column, it will be used for splitting '
-             '(0: train, 1: validation, 2: test), '
-             'otherwise the dataset will be randomly split'
+        "--dataset",
+        help="input data file path. "
+        "If it has a split column, it will be used for splitting "
+        "(0: train, 1: validation, 2: test), "
+        "otherwise the dataset will be randomly split",
     )
-    parser.add_argument('--training_set', help='input train data file path')
-    parser.add_argument(
-        '--validation_set', help='input validation data file path'
-    )
-    parser.add_argument('--test_set', help='input test data file path')
+    parser.add_argument("--training_set", help="input train data file path")
+    parser.add_argument("--validation_set", help="input validation data file path")
+    parser.add_argument("--test_set", help="input test data file path")
 
     parser.add_argument(
-        '--training_set_metadata',
-        help='input metadata JSON file path. An intermediate preprocessed file '
-             'containing the mappings of the input file created '
-             'the first time a file is used, in the same directory '
-             'with the same name and a .json extension'
+        "--training_set_metadata",
+        help="input metadata JSON file path. An intermediate preprocessed file "
+        "containing the mappings of the input file created "
+        "the first time a file is used, in the same directory "
+        "with the same name and a .json extension",
     )
 
     parser.add_argument(
-        '--data_format',
-        help='format of the input data',
-        default='auto',
-        choices=['auto', 'csv', 'excel', 'feather', 'fwf', 'hdf5',
-                 'html' 'tables', 'json', 'jsonl', 'parquet', 'pickle', 'sas',
-                 'spss', 'stata', 'tsv']
+        "--data_format",
+        help="format of the input data",
+        default="auto",
+        choices=[
+            "auto",
+            "csv",
+            "excel",
+            "feather",
+            "fwf",
+            "hdf5",
+            "html" "tables",
+            "json",
+            "jsonl",
+            "parquet",
+            "pickle",
+            "sas",
+            "spss",
+            "stata",
+            "tsv",
+        ],
     )
 
     parser.add_argument(
-        '-sspi',
-        '--skip_save_processed_input',
-        help='skips saving intermediate HDF5 and JSON files',
-        action='store_true',
-        default=False
+        "-sspi",
+        "--skip_save_processed_input",
+        help="skips saving intermediate HDF5 and JSON files",
+        action="store_true",
+        default=False,
     )
 
     # ----------------
@@ -273,146 +269,123 @@ def cli(sys_argv):
     # ----------------
     config = parser.add_mutually_exclusive_group(required=True)
     config.add_argument(
-        '-c',
-        '--config',
-        type=yaml.safe_load,
-        help='config'
+        "-c",
+        "--config",
+        type=load_yaml,
+        help="Path to the YAML file containing the model configuration",
     )
     config.add_argument(
-        '-cf',
-        '--config_file',
-        help='YAML file describing the model. Ignores --config'
+        "-cs",
+        "--config_str",
+        dest="config",
+        type=load_config_from_str,
+        help="JSON or YAML serialized string of the model configuration",
     )
 
+    parser.add_argument("-mlp", "--model_load_path", help="path of a pretrained model to load as initialization")
+    parser.add_argument("-mrp", "--model_resume_path", help="path of the model directory to resume training of")
     parser.add_argument(
-        '-mlp',
-        '--model_load_path',
-        help='path of a pretrained model to load as initialization'
-    )
-    parser.add_argument(
-        '-mrp',
-        '--model_resume_path',
-        help='path of the model directory to resume training of'
-    )
-    parser.add_argument(
-        '-sstd',
-        '--skip_save_training_description',
-        action='store_true',
+        "-sstd",
+        "--skip_save_training_description",
+        action="store_true",
         default=False,
-        help='disables saving the description JSON file'
+        help="disables saving the description JSON file",
     )
     parser.add_argument(
-        '-ssts',
-        '--skip_save_training_statistics',
-        action='store_true',
+        "-ssts",
+        "--skip_save_training_statistics",
+        action="store_true",
         default=False,
-        help='disables saving training statistics JSON file'
+        help="disables saving training statistics JSON file",
     )
     parser.add_argument(
-        '-ssm',
-        '--skip_save_model',
-        action='store_true',
+        "-ssm",
+        "--skip_save_model",
+        action="store_true",
         default=False,
-        help='disables saving weights each time the model improves. '
-             'By default Ludwig saves  weights after each epoch '
-             'the validation metric imrpvoes, but  if the model is really big '
-             'that can be time consuming. If you do not want to keep '
-             'the weights and just find out what performance a model can get '
-             'with a set of hyperparameters, use this parameter to skip it'
+        help="disables saving weights each time the model improves. "
+        "By default Ludwig saves  weights after each epoch "
+        "the validation metric (improves, but  if the model is really big "
+        "that can be time consuming. If you do not want to keep "
+        "the weights and just find out what performance a model can get "
+        "with a set of hyperparameters, use this parameter to skip it",
     )
     parser.add_argument(
-        '-ssp',
-        '--skip_save_progress',
-        action='store_true',
+        "-ssp",
+        "--skip_save_progress",
+        action="store_true",
         default=False,
-        help='disables saving weights after each epoch. By default ludwig saves '
-             'weights after each epoch for enabling resuming of training, but '
-             'if the model is really big that can be time consuming and will '
-             'save twice as much space, use this parameter to skip it'
+        help="disables saving weights after each epoch. By default ludwig saves "
+        "weights after each epoch for enabling resuming of training, but "
+        "if the model is really big that can be time consuming and will "
+        "save twice as much space, use this parameter to skip it",
     )
     parser.add_argument(
-        '-ssl',
-        '--skip_save_log',
-        action='store_true',
+        "-ssl",
+        "--skip_save_log",
+        action="store_true",
         default=False,
-        help='disables saving TensorBoard logs. By default Ludwig saves '
-             'logs for the TensorBoard, but if it is not needed turning it off '
-             'can slightly increase the overall speed'
+        help="disables saving TensorBoard logs. By default Ludwig saves "
+        "logs for the TensorBoard, but if it is not needed turning it off "
+        "can slightly increase the overall speed",
     )
 
     # ------------------
     # Runtime parameters
     # ------------------
     parser.add_argument(
-        '-rs',
-        '--random_seed',
+        "-rs",
+        "--random_seed",
         type=int,
         default=42,
-        help='a random seed that is going to be used anywhere there is a call '
-             'to a random number generator: data splitting, parameter '
-             'initialization and training set shuffling'
+        help="a random seed that is going to be used anywhere there is a call "
+        "to a random number generator: data splitting, parameter "
+        "initialization and training set shuffling",
+    )
+    parser.add_argument("-g", "--gpus", nargs="+", type=int, default=None, help="list of gpus to use")
+    parser.add_argument(
+        "-gml", "--gpu_memory_limit", type=int, default=None, help="maximum memory in MB to allocate per GPU device"
     )
     parser.add_argument(
-        '-g',
-        '--gpus',
-        nargs='+',
-        type=int,
-        default=None,
-        help='list of gpus to use'
+        "-dpt",
+        "--disable_parallel_threads",
+        action="store_false",
+        dest="allow_parallel_threads",
+        help="disable TensorFlow from using multithreading for reproducibility",
     )
     parser.add_argument(
-        '-gml',
-        '--gpu_memory_limit',
-        type=int,
-        default=None,
-        help='maximum memory in MB to allocate per GPU device'
+        "-b",
+        "--backend",
+        help="specifies backend to use for parallel / distributed execution, "
+        "defaults to local execution or Horovod if called using horovodrun",
+        choices=ALL_BACKENDS,
     )
     parser.add_argument(
-        '-dpt',
-        '--disable_parallel_threads',
-        action='store_false',
-        dest='allow_parallel_threads',
-        help='disable TensorFlow from using multithreading for reproducibility'
-    )
-    parser.add_argument(
-        '-uh',
-        '--use_horovod',
-        action='store_true',
-        default=None,
-        help='uses horovod for distributed training'
-    )
-    parser.add_argument(
-        '-dbg',
-        '--debug',
-        action='store_true',
-        default=False, help='enables debugging mode'
-    )
-    parser.add_argument(
-        '-l',
-        '--logging_level',
-        default='info',
-        help='the level of logging to use',
-        choices=['critical', 'error', 'warning', 'info', 'debug', 'notset']
+        "-l",
+        "--logging_level",
+        default="info",
+        help="the level of logging to use",
+        choices=["critical", "error", "warning", "info", "debug", "notset"],
     )
 
+    add_contrib_callback_args(parser)
     args = parser.parse_args(sys_argv)
 
+    args.callbacks = args.callbacks or []
+    for callback in args.callbacks:
+        callback.on_cmdline("train", *sys_argv)
+
     args.logging_level = logging_level_registry[args.logging_level]
-    logging.getLogger('ludwig').setLevel(
-        args.logging_level
-    )
+    logging.getLogger("ludwig").setLevel(args.logging_level)
     global logger
-    logger = logging.getLogger('ludwig.train')
+    logger = logging.getLogger("ludwig.train")
 
-    set_on_master(args.use_horovod)
-
-    if is_on_master():
-        print_ludwig('Train', LUDWIG_VERSION)
+    args.backend = initialize_backend(args.backend or args.config.get("backend"))
+    if args.backend.is_coordinator():
+        print_ludwig("Train", LUDWIG_VERSION)
 
     train_cli(**vars(args))
 
 
-if __name__ == '__main__':
-    contrib_import()
-    contrib_command("train", *sys.argv)
+if __name__ == "__main__":
     cli(sys.argv[1:])
