@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -35,7 +34,7 @@ class HTMLStripper(HTMLParser):
         self.fed.append(data)
 
     def get_data(self):
-        return ''.join(self.fed)
+        return "".join(self.fed)
 
     def error(self, message):
         logger.error(message)
@@ -48,29 +47,32 @@ def strip_tags(html):
 
 
 # regular expressions for cleaning text
-res_pre = [
-    (re.compile(r'([^.:;\?\!>])(<br/?>)'), r'\1.\2'),
-    (re.compile(r'<br/?>'), r' ')
-]
+res_pre = [(re.compile(r"([^.:;\?\!>])(<br/?>)"), r"\1.\2"), (re.compile(r"<br/?>"), r" ")]
 res_post = [
-    (re.compile(r'[ \t\0]'), r' '),
-    (re.compile(r'[–_]'), r'-'),
-    (re.compile(r'[\’\‘]'), r'''),
-    (re.compile(r'[”“]]'), r'''),
-    (re.compile(r'℅'), r'%'),
-    (re.compile(r'([^.>])(<br/?>)'), r'\1.\2'),
-    (re.compile(r'\\\\[NnRr]'), r' '),
-    (re.compile(r'\\[NnRr]'), r' '),
-    (re.compile(r'[\n\r]'), r' '),
-    (re.compile(r'\\\\'), r' / '),
-    (re.compile(r'<br/?>'), r' '),
-    (re.compile(r'\\\\'''), r'\''),
-    (re.compile(r'^\'([^\']+)$'), r'\1'),
-    (re.compile(r'([\<\>\{\}\[\]\(\)\-\+\=:;,\./\?\!\$%&£#@\'₹ ])\1+'), r'\1'),
-    (re.compile(
-        r'[^qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM1234567890\<\>\{\}\[\]\(\)\-\+\=:;,\./\?\!\$%&£#@\'₹ ]'),
-     r' '),
-    (re.compile(r'\s{2,}'), r' ')
+    (re.compile(r"[ \t\0]"), r" "),
+    (re.compile(r"[–_]"), r"-"),
+    (
+        re.compile(r"[\’\‘]"),
+        r"""),
+    (re.compile(r'[”“]]'), r""",
+    ),
+    (re.compile(r"℅"), r"%"),
+    (re.compile(r"([^.>])(<br/?>)"), r"\1.\2"),
+    (re.compile(r"\\\\[NnRr]"), r" "),
+    (re.compile(r"\\[NnRr]"), r" "),
+    (re.compile(r"[\n\r]"), r" "),
+    (re.compile(r"\\\\"), r" / "),
+    (re.compile(r"<br/?>"), r" "),
+    (re.compile(r"\\\\" ""), r"\'"),
+    (re.compile(r"^\'([^\']+)$"), r"\1"),
+    (re.compile(r"([\<\>\{\}\[\]\(\)\-\+\=:;,\./\?\!\$%&£#@\'₹ ])\1+"), r"\1"),
+    (
+        re.compile(
+            r"[^qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM1234567890\<\>\{\}\[\]\(\)\-\+\=:;,\./\?\!\$%&£#@\'₹ ]"  # noqa
+        ),
+        r" ",
+    ),
+    (re.compile(r"\s{2,}"), r" "),
 ]
 
 
